#include "AnimatedString.h"
#include <stdio.h>
#include <stdlib.h>
#include "mmsystem.h"

#define RANDOM(s,e) ((rand() % ((e)-(s))) + (s))

#define MOVE 4
#define STOP 5
#define EXPLOSION 6

AnimatedString::AnimatedString()
{
  chipList = NULL;
  numSlices = 0;
  slices = NULL;
  sourceDc = NULL;
  status = NOTREADY;
}

AnimatedString::~AnimatedString()
{
  if(chipList != NULL){
	list<Chip*>::iterator p;
	for(p = chipList->begin(); p != chipList->end(); p++)
	  delete *p;
	delete chipList;
  }
  delete slices;
}

int AnimatedString::Create(char *filename,int chipsize, RECT *screen, ChipsDrawing *drawingMode)
{
  if(sourceDc != NULL)
	return TRUE;
  HBITMAP hbm;
  BITMAP bm;

  if((hbm = (HBITMAP)LoadImage(NULL,filename,IMAGE_BITMAP,0,0,
					  LR_LOADFROMFILE | LR_CREATEDIBSECTION)) == NULL){
	return FALSE;
  }
  GetObject(hbm,sizeof(bm),&bm);
  bitmapRect.top = bitmapRect.left = 0;
  bitmapRect.right = bm.bmWidth;
  bitmapRect.bottom = bm.bmHeight;
  chipSize = chipsize;
  screenRect = *screen;
  sourceDc = CreateCompatibleDC(NULL);
  SelectObject(sourceDc,hbm);
  DeleteObject(hbm);

  chipsDrawing = drawingMode;
  int i;
  for(i = 0; i < bitmapRect.bottom; i++)
	if(GetPixel(sourceDc,0,i) == 0x000000)
	  numSlices++;

  slices = new RECT[numSlices+1];
  int count = 0;
  for(i = 0; i < bitmapRect.bottom; i++)
	if(GetPixel(sourceDc,0,i) == 0x000000){
	  slices[count].bottom = i;
	  if(count != numSlices)
		slices[count+1].top = i;
	  count++;
	}
  slices[0].top = 0;
  slices[count].bottom = bitmapRect.bottom;

  status = INITIAL;
  return TRUE;
}

int AnimatedString::InitSlice(int n)
{
  if(n > numSlices)
	return FALSE;

  if(chipList != NULL){
	list<Chip*>::iterator p;
	for(p = chipList->begin(); p != chipList->end(); p++)
	  delete *p;
	delete chipList;
  }

  // ʂ̍0x000000̐F̃rbg΁A܂ł
  // ̃XCXƂĒ`܂
  
  int fromX,fromY;
  int baseX,baseY;
  baseX = screenRect.right/2 - bitmapRect.right/2;
  baseY = screenRect.bottom/2 - (slices[n].bottom-slices[n].top)/2;
  if(chipsDrawing[n].initialMode == C_POINT){
	fromX = RANDOM(0,screenRect.right);
	fromY = RANDOM(0,screenRect.bottom);
  }

  chipList = new list<Chip*>;
  int x,y;
  for(y = slices[n].top; y < slices[n].bottom; y += chipSize)
	for(x = 1; x < bitmapRect.right; x += chipSize){
	  if(!IsWhite(sourceDc,x,y,chipSize)){
		if(chipsDrawing[n].initialMode != C_POINT){
		  switch(chipsDrawing[n].initialMode){
		  case C_RANDOM:
			fromX = RANDOM(0,screenRect.right);
			fromY = RANDOM(0,screenRect.bottom);
			break;
		  case C_TOP:
			fromX = RANDOM(0,screenRect.right);
			fromY = 1;
			break;
		  case C_BOTTOM:
			fromX = RANDOM(0,screenRect.right);
			fromY = screenRect.bottom - chipSize;
			break;
		  case C_RIGHT:
			fromX = screenRect.right - chipSize;
			fromY = RANDOM(0,screenRect.bottom);
			break;
		  case C_LEFT:
			fromX = 1;
			fromY = RANDOM(0,screenRect.bottom);
			break;
		  }
		}			
		Chip *chip = new Chip;
		chip->SetRect(x,y,x+chipSize,y+chipSize);
		chip->SetStartPoint(fromX,fromY);
		chip->SetEndPoint(x+baseX,y+baseY-slices[n].top);
		chipList->push_front(chip);
	  }
	}
  return TRUE;
}

int AnimatedString::DrawSlice(HDC dc, int percent)
{
  if(chipList == NULL)
	return FALSE;
  list<Chip*>::iterator p;
  RECT *base;
  POINT *drawPoint;
  for(p = chipList->begin(); p != chipList->end(); p++){
	base = (*p)->GetRect();
	drawPoint = (*p)->GetPoint(percent);
	BitBlt(dc,drawPoint->x,drawPoint->y,chipSize,chipSize,
		   sourceDc,base->left,base->top,SRCCOPY);
  }
  return TRUE;
}

// w͈͂ǂׂ
int AnimatedString::IsWhite(HDC dc, int x, int y, int chipsize)
{
  int i,j;
  int startx = x;
  int starty = y;
  int endx = x + chipsize;
  int endy = y + chipsize;

  for(j = starty; j < endy; j++)
	for(i = startx; i < endx; i++)
	  if(GetPixel(dc,i,j) != 0xFFFFFF)
		return FALSE;
  return TRUE;
}

int AnimatedString::Start()
{
  status = RUNNING;
  drawStatus = MOVE;
  currentSlice = 0;
  if(InitSlice(currentSlice) == FALSE){
	status = FINISHED;
	return FALSE;
  }
  lapTime = timeGetTime();
  return TRUE;
}

// ʂ𓮂֐
int AnimatedString::Move(HDC dc)
{
  // ĂȂEIĂꍇ̓^[
  if(status != RUNNING)
	return FALSE;

  // }Vp[ɂēςȂ悤ɁA
  // ԂvĒ߂܂B
  // ɂāA}Vł͂Ȃ߂炩ɓA
  // x}Vł͍r悤ɂȂ܂
  int newTime = timeGetTime();
  int elapseTime = newTime - lapTime;

  switch(drawStatus){
  case MOVE:
	// gݗĂɓĂ
	if(elapseTime >= chipsDrawing[currentSlice].moveTime){
	  lapTime = newTime;
	  drawStatus = STOP;
	}
	DrawSlice(dc,elapseTime*100/chipsDrawing[currentSlice].moveTime);
	break;
  case STOP:
	// ʂĂ
	if(elapseTime >= chipsDrawing[currentSlice].stopTime){
	  lapTime = newTime;
	  drawStatus = EXPLOSION;
	  if(currentSlice == numSlices){
		// Ō̃t[͔Ȃ悤ɂ܂
		status = FINISHED;
		DrawSlice(dc,100);
		return FALSE;
	  }
	}
	DrawSlice(dc,100);
	break;
  case EXPLOSION:
	// ʂɁAĂ
	if(elapseTime >= chipsDrawing[currentSlice].explosionTime){
	  lapTime = newTime;
	  drawStatus = MOVE;
	  currentSlice++;
	  // Ō̃XCX\͒~
	  if(InitSlice(currentSlice) == FALSE){
		status = FINISHED;
	  }
	}
	DrawSlice(dc,100 - elapseTime*100/chipsDrawing[currentSlice].explosionTime);
	break;
  }
  return TRUE;
}
